@@ -35,11 +35,10 @@ struct DualMismatchError{A,B} <: Exception
 end
 
 Base.showerror(io::IO, e::DualMismatchError{A,B}) where {A,B} =
-    print(io, "Cannot determine ordering of Dual tags $(e.a) and $(e.b)")
+    print(io, "Cannot determine ordering of Dual tags ", e.a, " and ", e.b)
 
 @noinline function throw_cannot_dual(V::Type)
-    throw(ArgumentError("Cannot create a dual over scalar type $V." *
-        " If the type behaves as a scalar, define ForwardDiff.can_dual(::Type{$V}) = true."))
+    throw(ArgumentError(lazy"Cannot create a dual over scalar type $V. If the type behaves as a scalar, define ForwardDiff.can_dual(::Type{$V}) = true."))
 end
 
 """
@@ -311,14 +310,9 @@ Base.eps(::Type{D}) where {D<:Dual} = eps(valtype(D))
 
 # The `base` keyword was added in Julia 1.8:
 # https://github.com/JuliaLang/julia/pull/42428
-if VERSION < v"1.8.0-DEV.725"
-    Base.precision(d::Dual) = precision(value(d))
-    Base.precision(::Type{D}) where {D<:Dual} = precision(valtype(D))
-else
-    Base.precision(d::Dual; base::Integer=2) = precision(value(d); base=base)
-    function Base.precision(::Type{D}; base::Integer=2) where {D<:Dual}
-        precision(valtype(D); base=base)
-    end
+Base.precision(d::Dual; base::Integer=2) = precision(value(d); base=base)
+function Base.precision(::Type{D}; base::Integer=2) where {D<:Dual}
+    precision(valtype(D); base=base)
 end
 
 function Base.nextfloat(d::ForwardDiff.Dual{T,V,N}) where {T,V,N}
@@ -349,11 +343,7 @@ Base.cld(x::Dual, y::Dual) = cld(value(x), value(y))
 
 Base.exponent(x::Dual) = exponent(value(x))
 
-if VERSION ≥ v"1.4"
-    Base.div(x::Dual, y::Dual, r::RoundingMode) = div(value(x), value(y), r)
-else
-    Base.div(x::Dual, y::Dual) = div(value(x), value(y))
-end
+Base.div(x::Dual, y::Dual, r::RoundingMode) = div(value(x), value(y), r)
 
 Base.hash(d::Dual, hsh::UInt) = hash(value(d), hsh)
 
@@ -400,13 +390,38 @@ for pred in UNARY_PREDICATES
     @eval Base.$(pred)(d::Dual) = $(pred)(value(d))
 end
 
-for pred in BINARY_PREDICATES
+# Before PR#481 this loop ran over this list:
+# BINARY_PREDICATES = Symbol[:isequal, :isless, :<, :>, :(==), :(<=), :(>=)]
+# Not a minimal set, as Base defines some in terms of others.
+@define_binary_dual_op(
+    Base.:(<),
+    (value(x) < value(y)) || (value(x) == value(y) && (partials(x) < partials(y))),
+    (value(x) < y) || (value(x) == y && (partials(x) < zero(partials(x)))),
+    (x < value(y)) || (x == value(y) && (zero(partials(y)) < partials(y))),
+)
+@define_binary_dual_op(
+    Base.:(<=),
+    (value(x) < value(y)) || (value(x) == value(y) && (partials(x) <= partials(y))),
+    (value(x) < y) || (value(x) == y && (partials(x) <= zero(partials(x)))),
+    (x < value(y)) || (x == value(y) && (zero(partials(y)) <= partials(y))),
+)
+
+@define_binary_dual_op(
+    Base.isless,
+    isless(value(x), value(y)) || (isequal(value(x), value(y)) && isless(partials(x), partials(y))),
+    isless(value(x), y)        || (isequal(value(x), y) && isless(partials(x), zero(partials(x)))),
+    isless(x, value(y))        || (isequal(x, value(y)) && isless(zero(partials(y)), partials(y))),
+)
+
+Base.iszero(x::Dual) = iszero(value(x)) && iszero(partials(x))  # shortcut, equivalent to x == zero(x)
+
+for pred in [:isequal, :(==)]
     @eval begin
         @define_binary_dual_op(
             Base.$(pred),
-            $(pred)(value(x), value(y)),
-            $(pred)(value(x), y),
-            $(pred)(x, value(y))
+            $(pred)(value(x), value(y)) && $(pred)(partials(x), partials(y)),
+            $(pred)(value(x), y)        && iszero(partials(x)),
+            $(pred)(x, value(y))        && iszero(partials(y)),
         )
     end
 end
@@ -689,6 +704,7 @@ end
 
 # sin/cos #
 #--------#
+
 function Base.sin(d::Dual{T}) where T
     s, c = sincos(value(d))
     return Dual{T}(s, c * partials(d))
@@ -707,13 +723,62 @@ end
 # sincospi #
 #----------#
 
-if VERSION >= v"1.6.0-DEV.292"
-    @inline function Base.sincospi(d::Dual{T}) where T
-        sd, cd = sincospi(value(d))
-        return (Dual{T}(sd, cd * π * partials(d)), Dual{T}(cd, -sd * π * partials(d)))
-    end
+@inline function Base.sincospi(d::Dual{T}) where T
+    sd, cd = sincospi(value(d))
+    return (Dual{T}(sd, cd * π * partials(d)), Dual{T}(cd, -sd * π * partials(d)))
 end
 
+# LinearAlgebra.givensAlgorithm #
+#-------------------------------#
+
+# This definition ensures that we match `LinearAlgebra.givensAlgorithm`
+# for non-dual numbers (i.e., `ForwardDiff.Dual` with zero partials)
+# `LinearAlgebra.givensAlgorithm` is derived from LAPACK's dlartg
+# which is [documented](https://netlib.org/lapack/explore-html/da/dd3/group__lartg_ga86f8f877eaea0386cdc2c3c175d9ea88.html) to return
+# three values c, s, u for two arguments x and y with
+# u = sgn(x) sqrt(x^2 + y^2)
+# c = x/u
+# s = y/u
+# The function is discontinuous in u at x=0
+@define_binary_dual_op(
+    LinearAlgebra.givensAlgorithm,
+    begin
+        vx, vy = value(x), value(y)
+        c, s, u = LinearAlgebra.givensAlgorithm(vx, vy)
+        ∂c∂x = s^2 / u
+        ∂c∂y = ∂s∂x = -(c * s / u)
+        ∂s∂y = c^2 / u
+        ∂x = partials(x)
+        ∂y = partials(y)
+        ∂c = _mul_partials(∂x, ∂y, ∂c∂x, ∂c∂y)
+        ∂s = _mul_partials(∂x, ∂y, ∂s∂x, ∂s∂y)
+        ∂u = _mul_partials(∂x, ∂y, c, s)
+        return Dual{Txy}(c, ∂c), Dual{Txy}(s, ∂s), Dual{Txy}(u, ∂u)
+    end,
+    begin
+        vx = value(x)
+        c, s, u = LinearAlgebra.givensAlgorithm(vx, y)
+        ∂c∂x = s^2 / u
+        ∂s∂x = -(c * s / u)
+        ∂x = partials(x)
+        ∂c = ∂c∂x * ∂x
+        ∂s = ∂s∂x * ∂x
+        ∂u = c * ∂x
+        return Dual{Tx}(c, ∂c), Dual{Tx}(s, ∂s), Dual{Tx}(u, ∂u)
+    end,
+    begin
+        vy = value(y)
+        c, s, u = LinearAlgebra.givensAlgorithm(x, vy)
+        ∂c∂y = -(c * s / u)
+        ∂s∂y = c^2 / u
+        ∂y = partials(y)
+        ∂c = ∂c∂y * ∂y
+        ∂s = ∂s∂y * ∂y
+        ∂u = s * ∂y
+        return Dual{Ty}(c, ∂c), Dual{Ty}(s, ∂s), Dual{Ty}(u, ∂u)
+    end,
+)
+
 # Symmetric eigvals #
 #-------------------#
 
@@ -814,6 +879,4 @@ for op in (:(Base.typemin), :(Base.typemax), :(Base.floatmin), :(Base.floatmax))
     end
 end
 
-if VERSION >= v"1.6.0-rc1"
-    Printf.tofloat(d::Dual) = Printf.tofloat(value(d))
-end
+Printf.tofloat(d::Dual) = Printf.tofloat(value(d))
