@@ -1,9 +1,11 @@
 module GradientTest
 
 import Calculus
+import LinearAlgebra
 import NaNMath
 
 using Test
+using LinearAlgebra
 using ForwardDiff
 using ForwardDiff: Dual, Tag
 using StaticArrays
@@ -60,7 +62,7 @@ cfgx = ForwardDiff.GradientConfig(sin, x)
 # test vs. Calculus.jl #
 ########################
 
-for f in DiffTests.VECTOR_TO_NUMBER_FUNCS
+@testset "$f" for f in DiffTests.VECTOR_TO_NUMBER_FUNCS
     v = f(X)
     g = ForwardDiff.gradient(f, X)
     @test isapprox(g, Calculus.gradient(f, X), atol=FINITEDIFF_ERROR)
@@ -147,12 +149,22 @@ end
 end
 
 @testset "exponential function at base zero" begin
-    @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0, -0.5]), [NaN, NaN])
-    @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0,  0.0]), [NaN, NaN])
-    @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0,  0.5]), [Inf, NaN])
+    if ForwardDiff.NANSAFE_MODE_ENABLED
+        @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0, -0.5]), [-Inf, -Inf])
+        @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0,  0.0]), [NaN, -Inf])
+        @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0,  0.5]), [Inf, 0.0])
+    else
+        @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0, -0.5]), [NaN, NaN])
+        @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0,  0.0]), [NaN, NaN])
+        @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0,  0.5]), [Inf, NaN])
+    end
     @test isequal(ForwardDiff.gradient(t -> t[1]^t[2], [0.0,  1.5]), [0.0, 0.0])
 end
 
+#############
+# bug fixes #
+#############
+
 # Issue 399
 @testset "chunk size zero" begin
     f_const(x) = 1.0
@@ -167,6 +179,7 @@ end
     @test_throws DimensionMismatch ForwardDiff.gradient(identity, fill(2pi, 10^6)) # chunk_mode_gradient
 end
 
+# Issue 548
 @testset "ArithmeticStyle" begin
     function f(p)
         sum(collect(0.0:p[1]:p[2]))
@@ -174,21 +187,99 @@ end
     @test ForwardDiff.gradient(f, [0.3, 25.0]) == [3486.0, 0.0]
 end
 
+@testset "det with branches" begin
+    # Issue 197
+    det2(A) = return (
+        A[1,1]*(A[2,2]*A[3,3]-A[2,3]*A[3,2]) -
+        A[1,2]*(A[2,1]*A[3,3]-A[2,3]*A[3,1]) +
+        A[1,3]*(A[2,1]*A[3,2]-A[2,2]*A[3,1])
+    )
+
+    A = [1 0 0; 0 2 0; 0 pi 3]
+    @test det2(A) == det(A) == 6
+    @test istril(A)
+
+    ∇A = [6 0 0; 0 3 -pi; 0 0 2]
+    @test ForwardDiff.gradient(det2, A) ≈ ∇A
+    @test ForwardDiff.gradient(det, A) ≈ ∇A
+
+    # And issue 407
+    @test ForwardDiff.hessian(det, A) ≈ ForwardDiff.hessian(det2, A)
+
+    # https://discourse.julialang.org/t/forwarddiff-and-zygote-return-wrong-jacobian-for-log-det-l/77961
+    S = [1.0 0.8; 0.8 1.0]
+    L = cholesky(S).L
+    @test ForwardDiff.gradient(L -> log(det(L)), Matrix(L)) ≈ [1.0 -1.3333333333333337; 0.0 1.666666666666667]
+    @test ForwardDiff.gradient(L -> logdet(L), Matrix(L)) ≈ [1.0 -1.3333333333333337; 0.0 1.666666666666667]
+end
+
 @testset "gradient for exponential with NaNMath" begin
-    @test isnan(ForwardDiff.gradient(x -> NaNMath.pow(x[1],x[1]), [NaN, 1.0])[1])
+    if ForwardDiff.NANSAFE_MODE_ENABLED
+        @test isequal(ForwardDiff.gradient(x -> NaNMath.pow(x[1],x[2]), [NaN, 1.0]), [1.0, NaN])
+    else
+        @test isequal(ForwardDiff.gradient(x -> NaNMath.pow(x[1],x[2]), [NaN, 1.0]), [NaN, NaN])
+    end
     @test ForwardDiff.gradient(x -> NaNMath.pow(x[1], x[2]), [1.0, 1.0]) == [1.0, 0.0]
     @test isnan(ForwardDiff.gradient((x) -> NaNMath.pow(x[1], x[2]), [-1.0, 0.5])[1])
 
-    @test isnan(ForwardDiff.gradient(x -> x[1]^x[2], [NaN, 1.0])[1])
+    if ForwardDiff.NANSAFE_MODE_ENABLED
+        @test isequal(ForwardDiff.gradient(x -> x[1]^x[2], [NaN, 1.0]), [1.0, NaN])
+    else
+        @test isequal(ForwardDiff.gradient(x -> x[1]^x[2], [NaN, 1.0]), [NaN, NaN])
+    end
     @test ForwardDiff.gradient(x -> x[1]^x[2], [1.0, 1.0]) == [1.0, 0.0]
     @test_throws DomainError ForwardDiff.gradient(x -> x[1]^x[2], [-1.0, 0.5])
 end
 
+@testset "branches in mul!" begin
+    a, b = rand(3,3), rand(3,3)
+
+    # Issue 536, version with 3-arg *, Julia 1.7:
+    @test ForwardDiff.derivative(x -> sum(x*a*b), 0.0) ≈ sum(a * b)
+
+    # version with just mul!
+    dx = ForwardDiff.derivative(0.0) do x
+        c = similar(a, typeof(x))
+        mul!(c, a, b, x, false)
+        sum(c)
+    end
+    @test dx ≈ sum(a * b)
+end
+
+# issue #738
+@testset "LowerTriangular, UpperTriangular and Diagonal" begin
+    for n in (3, 10, 20)
+        M = rand(n, n)
+        for T in (LowerTriangular, UpperTriangular, Diagonal)
+            @test ForwardDiff.gradient(sum, T(randn(n, n))) == T(ones(n, n))
+            @test ForwardDiff.gradient(x -> dot(M, x), T(randn(n, n))) == T(M)
+
+            # Check number of function evaluations and chunk sizes
+            fevals = Ref(0)
+            npartials = Ref(0)
+            y = ForwardDiff.gradient(T(randn(n, n))) do x
+                fevals[] += 1
+                npartials[] += ForwardDiff.npartials(eltype(x))
+                return sum(x)
+            end
+            if npartials[] <= ForwardDiff.DEFAULT_CHUNK_THRESHOLD
+                # Vector mode (single evaluation)
+                @test fevals[] == 1
+                @test npartials[] == sum(y)
+            else
+                # Chunk mode (multiple evaluations)
+                @test fevals[] > 1
+                @test sum(y) <= npartials[] < sum(y) + fevals[]
+            end
+        end
+    end
+end
+
 # issue #769
 @testset "functions with `Dual` output" begin
     x = [Dual{OuterTestTag}(Dual{TestTag}(1.3, 2.1), Dual{TestTag}(0.3, -2.4))]
     f(x) = sum(ForwardDiff.value, x)
-    der = ForwardDiff.derivative(ForwardDiff.value, first(x))
+    der = ForwardDiff.derivative(ForwardDiff.value, only(x))
 
     # Vector mode
     grad = ForwardDiff.gradient(f, x)
@@ -210,4 +301,50 @@ end
     @test grad == SVector{3}(der, der, der)
 end
 
+@testset "NaN-safe mode" begin
+    # issue #774
+    f = x -> log(zero(x[1]) + x[2])
+    x = [1.0, 0.0]
+    y1 = ForwardDiff.gradient(f, x)
+    y2 = ForwardDiff.gradient(f, x, ForwardDiff.GradientConfig(f, x, ForwardDiff.Chunk{1}()))
+    y3 = ForwardDiff.gradient(f, x, ForwardDiff.GradientConfig(f, x, ForwardDiff.Chunk{2}()))
+    for y in (y1, y2, y3)
+        if ForwardDiff.NANSAFE_MODE_ENABLED
+            @test y == [0.0, Inf]
+        else
+            @test isequal(y, [NaN, Inf])
+        end
+    end
+
+    # issue #745
+    g = a -> a[1] * exp(-a[2])
+    a = [1.0, -1e3]
+    b1 = ForwardDiff.gradient(g, a)
+    b2 = ForwardDiff.gradient(g, a, ForwardDiff.GradientConfig(g, a, ForwardDiff.Chunk{1}()))
+    b3 = ForwardDiff.gradient(g, a, ForwardDiff.GradientConfig(g, a, ForwardDiff.Chunk{2}()))
+    for b in (b1, b2, b3)
+        if ForwardDiff.NANSAFE_MODE_ENABLED
+            @test b == [Inf, -Inf]
+        else
+            @test isequal(b, [NaN, NaN])
+        end
+    end
+end
+
+@testset "Givens rotations: Gradients" begin
+    # Test different branches in `LinearAlgebra.givensAlgorithm`
+    for f in [randexp(), -randexp()], g in [0.0, f / 2, 2f, -f / 2, -2f], i in 1:3
+        # Gradients wrt to a single input argument
+        dydf = only(ForwardDiff.gradient(x -> LinearAlgebra.givensAlgorithm(only(x), g)[i], [f]))
+        @test dydf == ForwardDiff.derivative(x -> LinearAlgebra.givensAlgorithm(x, g)[i], f)
+        dydg = only(ForwardDiff.gradient(x -> LinearAlgebra.givensAlgorithm(f, only(x))[i], [g]))
+        @test dydg == ForwardDiff.derivative(x -> LinearAlgebra.givensAlgorithm(f, x)[i], g)
+
+        # Gradient with respect to both input arguments
+        grad = ForwardDiff.gradient(x -> LinearAlgebra.givensAlgorithm(x[1], x[2])[i], [f, g])
+        @test grad == [dydf, dydg]
+        @test grad ≈ Calculus.gradient(x -> LinearAlgebra.givensAlgorithm(x[1], x[2])[i], [f, g])
+    end
+end
+
 end # module
