@@ -232,6 +232,10 @@ for T in (StaticArrays.SArray, StaticArrays.MArray)
     @inferred ForwardDiff.JacobianConfig(f, sx)
 end
 
+#########
+# misc. #
+#########
+
 @testset "dimension errors for jacobian" begin
     @test_throws DimensionMismatch ForwardDiff.jacobian(identity, 2pi) # input
     @test_throws DimensionMismatch ForwardDiff.jacobian(sum, fill(2pi, 2)) # vector_mode_jacobian
@@ -249,27 +253,76 @@ end
     @test ForwardDiff.jacobian(ev2, x0) ≈ Calculus.finite_difference_jacobian(ev2, x0)
 
     x0_svector = SVector{2}(x0)
+    @test ForwardDiff.jacobian(ev1, x0_svector) isa SMatrix{2, 2}
     @test ForwardDiff.jacobian(ev1, x0_svector) ≈ Calculus.finite_difference_jacobian(ev1, x0)
 
     x0_mvector = MVector{2}(x0)
+    @test ForwardDiff.jacobian(ev1, x0_mvector) isa MMatrix{2, 2}
     @test ForwardDiff.jacobian(ev1, x0_mvector) ≈ Calculus.finite_difference_jacobian(ev1, x0)
 end
 
 @testset "type stability" begin
     g!(dy, y) = dy[1] = y[1]
     @inferred ForwardDiff.jacobian(g!, [1.0], [0.0])
+
+    @testset "issue 639" begin
+        f(x) = SA[x[1]^2+x[2]^2, x[2]^2+x[3]^2]
+        x = SA[1.0, 2.0, 3.0]
+        y = f(x)
+        imdr = DiffResults.JacobianResult(y, x)
+        @inferred ForwardDiff.jacobian!(imdr, f, x)
+    end
+
+    @testset "pr 735" begin
+        f(x) = x .^ 2 ./ 2
+        function withjacobian(x)
+            res = DiffResults.JacobianResult(x)
+            res = ForwardDiff.jacobian!(res, f, x)
+            return DiffResults.value(res), DiffResults.jacobian(res)
+        end
+        @inferred withjacobian(SA[1.0, 2.0])
+    end
+end
+
+# issues #436, #740
+@testset "BigFloat" begin
+    # Unassigned entries in the output
+    x = BigFloat.(1:9)
+    for chunksize in (1, 2, 9)
+        y = similar(x)
+        @test all(i -> !isassigned(y, i), eachindex(y))
+        cfg = ForwardDiff.JacobianConfig(copyto!, y, x, ForwardDiff.Chunk{chunksize}())
+        res = ForwardDiff.jacobian(copyto!, y, x, cfg)
+        @test y == x
+        @test res isa Matrix{BigFloat}
+        @test res == I
+    end
+
+    # Unassigned (but unused) entry in the input and unassigned entries in the output
+    resize!(x, 10)
+    f = (y, x) -> copyto!(y, 1, x, 1, 9)
+    for chunksize in (1, 2, 10)
+        y = similar(x, 9)
+        @test all(i -> !isassigned(y, i), eachindex(y))
+        cfg = ForwardDiff.JacobianConfig(f, y, x, ForwardDiff.Chunk{chunksize}())
+        res = ForwardDiff.jacobian(f, y, x, cfg)
+        @test y == x[1:(end-1)]
+        @test res isa Matrix{BigFloat}
+        @test res[:, 1:(end-1)] == I
+        @test all(iszero, res[:, end])
+    end
 end
 
 # issue #769
 @testset "functions with `Dual` output" begin
     x = [Dual{OuterTestTag}(Dual{TestTag}(1.3, 2.1), Dual{TestTag}(0.3, -2.4))]
     f(x) = map(ForwardDiff.value, x)
-    der = ForwardDiff.derivative(ForwardDiff.value, first(x))
+    der = ForwardDiff.derivative(ForwardDiff.value, only(x))
 
     # Vector mode
     jac = ForwardDiff.jacobian(f, x)
     @test jac isa Matrix{typeof(der)}
-    @test jac == fill(der, 1, 1)
+    @test jac == [der;;]
     jac = ForwardDiff.jacobian(f, SVector{1}(x))
     @test jac isa SMatrix{1,1,typeof(der)}
     @test jac == SMatrix{1,1}(der)
